#ifndef __EVOLUTIONCONTROLLER_H__
#define __EVOLUTIONCONTROLLER_H__

#include "ODEHuman.h"
#include "Timer.h"
#include <list>
#include <stdlib.h>

/*********************Defines********************************/
#define EVOLUTION_TIME 10 //in seconds; each person gets (at most) this much time
#define ACTIVATION_FUNCTION 0 //sine function
#define BODY_SIZE 1.3
#define FILE_WRITING_PERIOD 10 //write output files every 10 generations
#define CUTOFF_HEIGHT_1 5 //used for standing or walking or stepping
#define CUTOFF_HEIGHT_2 3 //used for jumping
#define UPRIGHTNESS_FACTOR 0 //stop if head gets this far ahead or behind torso
/************************************************************/

class EvolutionController 
{
public:
	EvolutionController(ODEWorld* theWorld, Timer* theTimer, point3d initialHumanPosition); 
	~EvolutionController();
	void EvolutionLoop(ODEHuman* humanPtr, float deltaTime);
	NeuralNet* GetFirstBrain();
	void GetUserInput();

private:  
	float FitnessFunction(ODEHuman* humanPtr, point3d start, point3d finish, int function);

	ODEWorld* world;
	Timer* timer;
	std::list<NeuralNet*> brainList;
	std::list<NeuralNet*>::iterator brainIter;
	float countDown;
	float distanceTraveled;
	point3d initialPosition;
	point3d finalPosition;
	int generationNumber;
	int fitnessType;
	float timeScale;
	int brainSize[3]; //# of hidden layers, number of neurons per layer
	FILE* genesFile;
	FILE* statsFile;
	int populationSize;
	char useInputFile;
	float averageHeight;
	float numberOfCycles;
	float cutoffHeight; //trials end when person falls below this height (or EVOLUTION_TIME time runs out)
	float highestHeight; //for jumping
	dReal numberOfStepsTaken; //for walking
	dReal previousLeftFootHeight;
	dReal previousRightFootHeight;
	char lastFootThatStepped; //for walking
	dReal steppingThreshold; //for walking
	NeuralNet* tempGenesStorage[FILE_WRITING_PERIOD];
	std::string tempStatsStorage[FILE_WRITING_PERIOD];
};

bool ComparisonFunction(NeuralNet* a, NeuralNet* b);

#define INTSIZE 10
char *itoa(int value);

#endif
